// IOCTLs
#define IOCTL_XENPCI_REGISTER_EVENT_HANDLER 0x803
#define IOCTL_XENPCI_UNREGISTER_EVENT_HANDLER 0x805
#define IOCTL_XENPCI_READ_PARAMETER 0x80c
#define IOCTL_XENPCI_WRITE_PARAMETER 0x811
#define IOCTL_XENPCI_DELETE_PARAM 0x812
#define IOCTL_XENPCI_SEND_MESSAGE 0x80d
#define IOCTL_XENPCI_GET_PARAM_COUNT 0x80e
#define IOCTL_XENPCI_GET_PARAM_BY_INDEX 0x80f
#define IOCTL_XENPCI_GET_ALL_PARAM_NAMES 0x820
#define IOCTL_XENPCI_GET_PARAM_VALUE_SIZE_BY_NAME 0x822
#define IOCTL_XENPCI_MODIFY_EVENT_FILTER 0x823
#define IOCTL_XENPCI_POST_EVENT 0x824
#define IOCTL_XENPCI_GET_EVENT_HEADER 0x825
#define IOCTL_XENPCI_GET_NEXT_EVENT_HEADER 0x826
#define IOCTL_XENPCI_GET_EVENT 0x827
#define IOCTL_XENPCI_GET_NEXT_EVENT 0x806
#define IOCTL_XENPCI_DISCARD_EVENT 0x828
#define IOCTL_XENPCI_DISCARD_NEXT_EVENT 0x829

// Definitions
#define OVMM_MAX_CHARS_PER_SEQUENCE		2048
#define OVMM_MAX_NAME_LEN				256
#define OVMM_MAX_VALUE_LEN				8192


// Data Structures

typedef struct _parameter
{
	struct list_head list;
    char *name;
    unsigned long name_size;
    char *value;
    unsigned long value_size;
} parameter_t;


typedef struct _application_entry
{
	struct list_head list;
    int registered;       
    unsigned long event_mask;
 	struct list_head events_list;
    wait_queue_head_t event_waitqueue;
    struct fasync_struct *async_queue;

}application_entry_t;


struct ovmapi_information
{
    unsigned long parameter_count;
    struct list_head parameter_list;
    struct mutex parameter_mutex;
    struct list_head registered_apps_list;
    struct mutex apps_list_mutex;
    unsigned long last_read_message; 
    unsigned long last_write_message; 
    struct xenbus_watch dom0_message_watch;
	unsigned long event_counter;
};

/*************************************************************/
/*            User space command headers                     */
/*************************************************************/
#define OVMAPI_EVT_PHASE_PRE 0x0000
#define OVMAPI_EVT_PHASE_IMMED 0x0001
#define OVMAPI_EVT_PHASE_POST 0x0002

#define OVMAPI_EVT_SEVERITY_DEBUG 0x0000
#define OVMAPI_EVT_SEVERITY_INFO 0x0001
#define OVMAPI_EVT_SEVERITY_WARNING 0x0002
#define OVMAPI_EVT_SEVERITY_ERROR 0x0003
#define OVMAPI_EVT_SEVERITY_CRITICAL 0x0004
#define OVMAPI_EVT_SEVERITY_SYSTEM 0x0005

#define OVMAPI_EVT_SNAPSHOT 0x001 
#define OVMAPI_EVT_MIGRATE 0x002
#define OVMAPI_EVT_SHUTDOWN 0x004
#define OVMAPI_EVT_RECONFIG 0x008
#define OVMAPI_EVT_IP_ADDRESS 0x010
#define OVMAPI_EVT_USER 0x20
#define OVMAPI_EVT_NEW_PARAM 0x40
#define OVMAPI_EVT_SYSTEM 0x80

#define OVMAPI_MAX_PROC_NAME_LEN 256
#define OVMAPI_MAX_SUBSYS_NAME_LEN 256
#define OVMAPI_MAX_USER_DATA_LEN 4096

// value size includes null terminator

struct param_message {
        u32 index;
        char name[OVMM_MAX_NAME_LEN];
        u32 value_size; 
        char *value;
}__attribute__((packed));


struct param_names {
    u32 total_names;
    char *name_entries; // each name is OVMM_MAX_NAME_LEN long.
}__attribute__((packed));

typedef struct _user_event_header
{
    unsigned long event_id;
    unsigned short type;
    unsigned short severity;
	unsigned short phase;
    unsigned short size; /* This is the size of real payload(ovmapi_event_t) */
} __attribute__((packed)) user_event_header_t;

#define OVMAPI_EVENT_MAXSIZE		(sizeof(user_event_header_t))
#define OVMAPI_EVENT_DATA_MAXSIZE	(4096 - sizeof(user_event_header_t))

// Special event types
#define OVMAPI_EVT_MORE_PROCESSING	0x4000

typedef struct _ovmapi_event_more_processing
{
    user_event_header_t header;
    unsigned long event_mask;
	char data[OVMAPI_EVENT_DATA_MAXSIZE - sizeof(unsigned long)];

} __attribute__((packed)) ovmapi_event_more_processing_t;

typedef struct _ovmapi_event
{
    user_event_header_t header;
	char payload[OVMAPI_EVENT_DATA_MAXSIZE];

} __attribute__((packed)) ovmapi_event_t;


typedef struct _ovmapi_event_list
{
    struct list_head list;
    ovmapi_event_t event_entry;

} ovmapi_event_list_t;

typedef struct _ovmapi_event_subscription
{
    unsigned long subscribe;
    unsigned long unsubscribe;
} __attribute__((packed)) ovmapi_event_subscription_t;

